@@ -10,7 +10,11 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2012-2013 Los Alamos National Security, Inc.  All rights reserved. 
+ * Copyright (c) 2012-2013 Los Alamos National Security, Inc.  All rights reserved.
+ * Copyright (c) 2014-2018 Intel, Inc. All rights reserved.
+ * Copyright (c) 2015      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2017      IBM Corporation. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -31,16 +35,16 @@
  */
 bool opal_dss_initialized = false;
 int opal_dss_verbose = -1;  /* by default disabled */
-int opal_dss_initial_size;
-int opal_dss_threshold_size;
-opal_pointer_array_t opal_dss_types;
-opal_data_type_t opal_dss_num_reg_types;
-opal_dss_buffer_type_t default_buf_type = OPAL_DSS_BUFFER_NON_DESC;
+int opal_dss_initial_size = -1;
+int opal_dss_threshold_size = -1;
+opal_pointer_array_t opal_dss_types = {{0}};
+opal_data_type_t opal_dss_num_reg_types = {0};
+static opal_dss_buffer_type_t default_buf_type = OPAL_DSS_BUFFER_NON_DESC;
 
 /* variable group id */
 static int opal_dss_group_id = -1;
 
-mca_base_var_enum_value_t buffer_type_values[] = {
+static mca_base_var_enum_value_t buffer_type_values[] = {
     {OPAL_DSS_BUFFER_NON_DESC, "non-described"},
     {OPAL_DSS_BUFFER_FULLY_DESC, "described"},
     {0, NULL}
@@ -71,12 +75,21 @@ static void opal_value_construct(opal_value_t* ptr)
 {
     ptr->key = NULL;
     ptr->type = OPAL_UNDEF;
+    memset(&ptr->data, 0, sizeof(ptr->data));
 }
 static void opal_value_destruct(opal_value_t* ptr)
 {
     if (NULL != ptr->key) {
         free(ptr->key);
     }
+    if (OPAL_STRING == ptr->type &&
+        NULL != ptr->data.string) {
+        free(ptr->data.string);
+    }
+    if (OPAL_BYTE_OBJECT == ptr->type &&
+        NULL != ptr->data.bo.bytes) {
+        free(ptr->data.bo.bytes);
+    }
 }
 OBJ_CLASS_INSTANCE(opal_value_t,
                    opal_list_item_t,
@@ -144,6 +157,7 @@ static void opal_pstat_construct(opal_pstats_t *obj)
     obj->time.tv_usec = 0;
     obj->priority = -1;
     obj->num_threads = -1;
+    obj->pss = 0.0;
     obj->vsize = 0.0;
     obj->rss = 0.0;
     obj->peak_vsize = 0.0;
@@ -218,6 +232,26 @@ OBJ_CLASS_INSTANCE(opal_node_stats_t, opal_object_t,
                    opal_node_stats_destruct);
 
 
+static void opal_envar_construct(opal_envar_t *obj)
+{
+    obj->envar = NULL;
+    obj->value = NULL;
+    obj->separator = '\0';
+}
+static void opal_envar_destruct(opal_envar_t *obj)
+{
+    if (NULL != obj->envar) {
+        free(obj->envar);
+    }
+    if (NULL != obj->value) {
+        free(obj->value);
+    }
+}
+OBJ_CLASS_INSTANCE(opal_envar_t,
+                   opal_list_item_t,
+                   opal_envar_construct,
+                   opal_envar_destruct);
+
 int opal_dss_register_vars (void)
 {
     mca_base_var_enum_t *new_enum;
@@ -267,7 +301,7 @@ int opal_dss_register_vars (void)
         return ret;
     }
 
-    /* the threshold as to where to stop doubling the size of the buffer 
+    /* the threshold as to where to stop doubling the size of the buffer
      * allocated memory and start doing additive increases */
     opal_dss_threshold_size = OPAL_DSS_DEFAULT_THRESHOLD_SIZE;
     ret = mca_base_var_register ("opal", "dss", NULL, "buffer_threshold_size", NULL,
@@ -535,6 +569,16 @@ int opal_dss_open(void)
                                                      "OPAL_FLOAT", &tmp))) {
         return rc;
     }
+    tmp = OPAL_DOUBLE;
+    if (OPAL_SUCCESS != (rc = opal_dss.register_type(opal_dss_pack_double,
+                                                     opal_dss_unpack_double,
+                                                     (opal_dss_copy_fn_t)opal_dss_std_copy,
+                                                     (opal_dss_compare_fn_t)opal_dss_compare_double,
+                                                     (opal_dss_print_fn_t)opal_dss_print_double,
+                                                     OPAL_DSS_UNSTRUCTURED,
+                                                     "OPAL_DOUBLE", &tmp))) {
+        return rc;
+    }
     tmp = OPAL_TIMEVAL;
     if (OPAL_SUCCESS != (rc = opal_dss.register_type(opal_dss_pack_timeval,
                                                      opal_dss_unpack_timeval,
@@ -544,6 +588,72 @@ int opal_dss_open(void)
                                                      OPAL_DSS_UNSTRUCTURED,
                                                      "OPAL_TIMEVAL", &tmp))) {
         return rc;
+    }
+     tmp = OPAL_TIME;
+    if (OPAL_SUCCESS != (rc = opal_dss.register_type(opal_dss_pack_time,
+                                                     opal_dss_unpack_time,
+                                                     (opal_dss_copy_fn_t)opal_dss_std_copy,
+                                                     (opal_dss_compare_fn_t)opal_dss_compare_time,
+                                                     (opal_dss_print_fn_t)opal_dss_print_time,
+                                                     OPAL_DSS_UNSTRUCTURED,
+                                                     "OPAL_TIME", &tmp))) {
+        return rc;
+    }
+
+    tmp = OPAL_NAME;
+    if (OPAL_SUCCESS != (rc = opal_dss.register_type(opal_dss_pack_name,
+                                          opal_dss_unpack_name,
+                                          (opal_dss_copy_fn_t)opal_dss_copy_name,
+                                          (opal_dss_compare_fn_t)opal_dss_compare_name,
+                                          (opal_dss_print_fn_t)opal_dss_print_name,
+                                          OPAL_DSS_UNSTRUCTURED,
+                                          "OPAL_NAME", &tmp))) {
+        return rc;
+    }
+
+    tmp = OPAL_JOBID;
+    if (OPAL_SUCCESS != (rc = opal_dss.register_type(opal_dss_pack_jobid,
+                                          opal_dss_unpack_jobid,
+                                          (opal_dss_copy_fn_t)opal_dss_copy_jobid,
+                                          (opal_dss_compare_fn_t)opal_dss_compare_jobid,
+                                          (opal_dss_print_fn_t)opal_dss_print_jobid,
+                                          OPAL_DSS_UNSTRUCTURED,
+                                          "OPAL_JOBID", &tmp))) {
+        return rc;
+    }
+
+    tmp = OPAL_VPID;
+    if (OPAL_SUCCESS != (rc = opal_dss.register_type(opal_dss_pack_vpid,
+                                          opal_dss_unpack_vpid,
+                                          (opal_dss_copy_fn_t)opal_dss_copy_vpid,
+                                          (opal_dss_compare_fn_t)opal_dss_compare_vpid,
+                                          (opal_dss_print_fn_t)opal_dss_print_vpid,
+                                          OPAL_DSS_UNSTRUCTURED,
+                                          "OPAL_VPID", &tmp))) {
+        return rc;
+    }
+
+
+    tmp = OPAL_STATUS;
+    if (OPAL_SUCCESS != (rc = opal_dss.register_type(opal_dss_pack_status,
+                                          opal_dss_unpack_status,
+                                          (opal_dss_copy_fn_t)opal_dss_std_copy,
+                                          (opal_dss_compare_fn_t)opal_dss_compare_status,
+                                          (opal_dss_print_fn_t)opal_dss_print_status,
+                                          OPAL_DSS_UNSTRUCTURED,
+                                          "OPAL_STATUS", &tmp))) {
+        return rc;
+    }
+
+    tmp = OPAL_ENVAR;
+    if (OPAL_SUCCESS != (rc = opal_dss.register_type(opal_dss_pack_envar,
+                                          opal_dss_unpack_envar,
+                                          (opal_dss_copy_fn_t)opal_dss_copy_envar,
+                                          (opal_dss_compare_fn_t)opal_dss_compare_envar,
+                                          (opal_dss_print_fn_t)opal_dss_print_envar,
+                                          OPAL_DSS_UNSTRUCTURED,
+                                          "OPAL_ENVAR", &tmp))) {
+        return rc;
     }
     /* All done */
 
