@@ -1,3 +1,4 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
@@ -5,18 +6,22 @@
  * Copyright (c) 2004-2005 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2010      Oak Ridge National Laboratory.  
+ * Copyright (c) 2010      Oak Ridge National Laboratory.
  *                         All rights reserved.
- * Copyright (c) 2010      Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2010-2017 Cisco Systems, Inc.  All rights reserved
  * Copyright (c) 2010      IBM Corporation.  All rights reserved.
+ * Copyright (c) 2014      Los Alamos National Security, LLC. All rights
+ *                         reserved.
+ * Copyright (c) 2018      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
@@ -27,18 +32,52 @@
 #include <stdlib.h>
 #include <dirent.h>
 
+#include <sys/param.h>
+#ifdef HAVE_SYS_MOUNT_H
+#include <sys/mount.h>
+#endif
+#ifdef HAVE_SYS_STATFS_H
+#include <sys/statfs.h>
+#endif
+#ifdef HAVE_SYS_VFS_H
+#include <sys/vfs.h>
+#endif
+
 #include "support.h"
 #include "opal/util/path.h"
 #include "opal/util/output.h"
 
-/*
 #define DEBUG
-*/
+
+/*
+ * On Linux, if this test is run with no command line params, it will
+ * run "mount" and analyze the output.  It will slurp up all mount
+ * points and figure out if they are network mounts or not.  Then it
+ * will send each of the mount points through opal_path_nfs() and
+ * compare the answer it gets to the answer it slurped from the output
+ * of "mount".
+ *
+ * On all platforms, if you provide one or more command line options,
+ * each command line option is given to opal_path_nfs() and the result
+ * is sent to stdout.
+ */
 
 #if !defined(__linux__)
-/* This test currently only works on Linux */
 int main(int argc, char* argv[])
 {
+    if (argc > 1) {
+        int i;
+
+        printf("Interactive opal_path_nfs() test:\n");
+        for (i = 1; i < argc; i++) {
+            printf ("Is dir[%d]:%s one of the detected network file systems? %s\n",
+                    i, argv[i], opal_path_nfs (argv[i], NULL) ? "Yes": "No");
+        }
+
+        return 0;
+    }
+
+    printf("No filename was given; nothing to do\n");
     return 77;
 }
 
@@ -65,7 +104,7 @@ int main(int argc, char* argv[])
         int i;
         for (i = 1; i < argc; i++)
             printf ("Is dir[%d]:%s one of the detected network file systems? %s\n",
-                    i, argv[i], opal_path_nfs (argv[i]) ? "Yes": "No");
+                    i, argv[i], opal_path_nfs (argv[i], NULL) ? "Yes": "No");
     }
 
     get_mounts (&num_dirs, &dirs, &nfs);
@@ -83,8 +122,8 @@ void test(char* file, bool expect)
 #ifdef DEBUG
     printf ("test(): file:%s bool:%d\n",
             file, expect);
-#endif 
-    if (expect == opal_path_nfs (file)) {
+#endif
+    if (expect == opal_path_nfs (file, NULL)) {
         test_success();
     } else {
         char * msg;
@@ -97,7 +136,6 @@ void test(char* file, bool expect)
 
 void get_mounts (int * num_dirs, char ** dirs[], bool * nfs[])
 {
-#define MAX_DIR 256
 #define SIZE 1024
     char * cmd = "mount | cut -f3,5 -d' ' > opal_path_nfs.out";
     int rc;
@@ -106,6 +144,7 @@ void get_mounts (int * num_dirs, char ** dirs[], bool * nfs[])
     char ** dirs_tmp;
     bool * nfs_tmp;
     char buffer[SIZE];
+    struct statfs mystatfs;
 
     rc = system (cmd);
 
@@ -114,17 +153,38 @@ void get_mounts (int * num_dirs, char ** dirs[], bool * nfs[])
         **dirs = NULL;
         *nfs = NULL;
     }
-    dirs_tmp = (char**) malloc (MAX_DIR * sizeof(char**));
-    nfs_tmp = (bool*) malloc (MAX_DIR * sizeof(bool));
 
+    /* First, count how many mount points there are.  Previous
+       versions of this test tried to have a (large) constant-sized
+       array for the mount points, but periodically it would break
+       because we would run this test on a system with a larger number
+       of mount points than the array.  So just count and make sure to
+       have an array large enough. */
     file = fopen("opal_path_nfs.out", "r");
+    int count = 0;
+    while (NULL != fgets (buffer, SIZE, file)) {
+        ++count;
+    }
+    printf("Found %d mounts\n", count);
+
+    // Add one more so we can have a NULL entry at the end
+    ++count;
+
+    dirs_tmp = (char**) calloc (count, sizeof(char*));
+    nfs_tmp = (bool*) calloc (count, sizeof(bool));
+
     i = 0;
     rc = 4711;
-    while (NULL != fgets (buffer, SIZE, file)) {
+    rewind(file);
+    // i should never be more than count, but be safe anyway.
+    while (i < count && NULL != fgets (buffer, SIZE, file)) {
         int mount_known;
         char fs[MAXNAMLEN];
 
-        dirs_tmp[i] = malloc (MAXNAMLEN);
+        if (!dirs_tmp[i]) {
+            dirs_tmp[i] = malloc (MAXNAMLEN);
+        }
+
         if (2 != (rc = sscanf (buffer, "%s %s\n", dirs_tmp[i], fs))) {
             goto out;
         }
@@ -138,12 +198,23 @@ void get_mounts (int * num_dirs, char ** dirs[], bool * nfs[])
             continue;
         }
 
+        /* Per https://svn.open-mpi.org/trac/ompi/ticket/4767, Linux
+           lies about fuse filesystems.  Skip them. */
+        if (0 == strncasecmp(fs, "fuse.", 5)) {
+            continue;
+        }
+
         /* If we get an fs type of "none", skip it (e.g.,
            http://www.open-mpi.org/community/lists/devel/2012/09/11493.php) */
         if (0 == strcasecmp(fs, "none")) {
             continue;
         }
 
+        /* If we can not stat the fs, skip it */
+        if (statfs (dirs_tmp[i], &mystatfs)) {
+            continue;
+        }
+
         /*
          * Later mounts override earlier mounts
          * (e.g. nfs-mount of /, vs. initial rootfs-mount of /).
